"""
Block data encryption using
"""

import logging
import secrets
from io import BytesIO, TextIOBase

__all__ = ("encrypt_data", "decrypt_data", "EncryptError", "DecryptError")

from typing import Union

# Magic bytes to identify an encrypted files
MAGIC_BYTES = b"SECF"  # "Secure Encrypted File"
ITERATIONS = 1800000
AES_KEY_LENGTH = 32  # max bytes for AES256

logger = logging.getLogger(__file__)


class DecryptError(ValueError):
    pass


class EncryptError(ValueError):
    pass


try:
    from Crypto.Cipher import AES
    from Crypto.Hash import SHA256
    from Crypto.Protocol.KDF import PBKDF2

    def _pad(data: bytes) -> bytes:
        """
        Pad data to be a multiple of 16 bytes (AES block size)
        """
        padding_length = 16 - (len(data) % 16)
        padding = bytes([padding_length] * padding_length)
        return data + padding

    def _unpad(data: bytes) -> bytes:
        """
        Check and remove PKCS7 padding
        """
        padding_length = data[-1]
        if padding_length < 1 or padding_length > AES.block_size:
            raise ValueError("Invalid padding length")
        if data[-padding_length:] != bytes([padding_length]) * padding_length:
            raise ValueError("Invalid padding bytes")
        return data[:-padding_length]

    def generate_key_from_password(
        password: str, salt: bytes = None
    ) -> tuple[bytes, bytes]:
        """
        Generate an AES key from a password using PBKDF2
        Returns the key and salt used
        """
        if salt is None:
            salt = secrets.token_bytes(16)

        key = PBKDF2(
            password,
            salt,
            dkLen=AES_KEY_LENGTH,  # AES-256 key size
            count=ITERATIONS,  # High iteration count for security
            hmac_hash_module=SHA256,
        )
        return key, salt

    def encrypt_data(
        input_stream: Union[BytesIO, TextIOBase], password: str, encoding: str = "utf-8"
    ) -> BytesIO:
        """
        Encrypt a file using AES-256 in CBC mode with a password-derived key
        """
        if not isinstance(input_stream, BytesIO):
            input_stream.seek(0)
            input_stream = BytesIO(input_stream.read().encode(encoding))
        first_bytes = input_stream.read(len(MAGIC_BYTES))
        if first_bytes == MAGIC_BYTES:
            logger.debug("Attempted to encrypt an already encrypted stream")
            raise EncryptError("This data is already encrypted")
        input_stream.seek(0)

        if not password:
            logger.debug("No or blank password provided")
            raise EncryptError("No or blank password provided")

        key, salt = generate_key_from_password(password)

        # Generate random IV
        iv = secrets.token_bytes(16)

        try:
            # Create cipher
            cipher = AES.new(key, AES.MODE_CBC, iv)

            # Pad and encrypt the data
            encrypted_data = cipher.encrypt(_pad(input_stream.getvalue()))
        except ValueError as exc:
            raise DecryptError(*exc.args) from exc

        logger.debug(f"Encryption successful ({len(encrypted_data)} + 36 bytes)")

        # Write magic bytes, salt, IV, and encrypted data
        return BytesIO(MAGIC_BYTES + salt + iv + encrypted_data)

    def decrypt_data(input_stream: BytesIO, password: str) -> BytesIO:
        """
        Decrypt data that was encrypted using encrypt_data()
        """
        # Read the magic bytes, salt, IV, and encrypted data
        magic = input_stream.read(len(MAGIC_BYTES))
        if magic != MAGIC_BYTES:
            logger.debug("Attempted to decrypt a non-encrypted stream")
            raise DecryptError("This data does not look to be encrypted")
        salt = input_stream.read(16)  # salt
        iv = input_stream.read(16)  # IV
        encrypted_data = input_stream.read()

        # Regenerate the key using the same password and salt
        key, _ = generate_key_from_password(password, salt)

        # Create cipher
        try:
            cipher = AES.new(key, AES.MODE_CBC, iv)
            padded_decrypted_data = cipher.decrypt(encrypted_data)
            # Decrypt and unpad the data
            decrypted_data = _unpad(padded_decrypted_data)
        except ValueError as e:
            raise DecryptError("Incorrect password or invalid data") from e
        logger.debug(f"Decryption successful ({len(decrypted_data)} bytes)")
        return BytesIO(decrypted_data)


except ImportError:

    def encrypt_data(_input_stream: BytesIO, _password: str) -> BytesIO:
        raise EncryptError("Encryption not supported")

    def decrypt_data(_input_stream: BytesIO, _password: str) -> BytesIO:
        raise DecryptError("Decryption not supported")



# -*- coding: utf-8 -*-
import os
import sys
import contextlib
from io import TextIOBase, BytesIO
from pathlib import Path
from string import Template
from typing import Dict, List, MutableMapping, Union, Optional, ContextManager, BinaryIO

from .env_crypto import decrypt_data, DecryptError

__all__ = (
    "load_env",
    "load_stream",
    "load_dotenv",  # alias
    "update_env",
    "unquote",
)


DEFAULT_ENVKEY = "DOTENV"
DEFAULT_DOTENV = ".env"
ENCRYPTED_EXT = ".enc"
DEFAULT_ENCODING = "utf-8"


def unquote(line, quotes="\"'"):
    if line and line[0] in quotes and line[-1] == line[0]:
        line = line[1:-1]
    return line


def update_env(env: MutableMapping[str, str], mapping: Dict):
    for k, v in mapping.items():
        env[str(k)] = str(v)


def _env_default(
    environ: MutableMapping[str, str], key: str, val: str, overwrite: bool = False
):
    if key and val and (overwrite or key not in environ):
        environ[key] = val


def _env_export(
    environ: MutableMapping[str, str], key: str, val: str, overwrite: bool = False
):
    if key and val and (overwrite or key not in environ):
        environ[key] = val
        os.environ[key] = val


def _env_files(
    env_file: str, search_path: List[Path], parents: bool, decrypt: bool, errors: bool
) -> List[str]:
    """expand env_file with the full search path, optionally parents as well"""

    def resolve_file(base_path: Path, name: str, _decrypt: bool) -> Optional[str]:
        """Returns the path to the env file, prioritising the encrypted version if enabled"""
        if _decrypt:
            encrypted_path = os.path.join(base_path, name + ENCRYPTED_EXT)
            if os.access(encrypted_path, os.R_OK):
                return encrypted_path

        standard_path = os.path.join(base_path, name)
        return standard_path if os.access(standard_path, os.R_OK) else None

    searched = []
    for path in search_path:
        path = path.resolve()
        if not path.is_dir():
            path = path.parent
        searched.append(path)

        for sub_path in [path] + list(path.parents):
            if env_path := resolve_file(sub_path, env_file, decrypt):
                yield env_path
                break
            elif not parents:
                break

    if errors:
        raise FileNotFoundError(f"{env_file} in {[s.as_posix() for s in searched]}")
    else:
        yield env_file


@contextlib.contextmanager
def open_env(path: Union[str, Path]) -> ContextManager[BinaryIO]:
    """same as open, allow monkeypatch"""
    fp = open(path, "rb")
    try:
        yield fp
    finally:
        fp.close()


ENV_COMMANDS = {
    "export": _env_export,
}


def _process_line(_lineno: int, string: str, errors: bool, _env_path: Path | None):
    """process a single line"""
    _func, _key, _val = _env_default, None, None
    parts = string.split("=", 1)
    if len(parts) == 2:
        _key, _val = parts
    elif len(parts) == 1:
        _key = parts[0]
    if _key:
        words = _key.split(maxsplit=1)
        if len(words) > 1:
            command, _key = words
            try:
                _func = ENV_COMMANDS[command]
            except KeyError:
                if errors:
                    path = _env_path.as_posix() if _env_path else "stream"
                    print(
                        f"unknown command {command} {path}({_lineno})",
                        file=sys.stderr,
                    )
    return _func, unquote(_key), unquote(_val)


def _process_stream(
    stream: BytesIO, environ, overwrite, errors, encoding=DEFAULT_ENCODING, env_path=None
):
    for lineno, line in enumerate(stream.readlines(), start=1):
        line = line.decode(encoding).strip()
        if line and line[0] != "#":
            func, key, val = _process_line(lineno, line, errors, env_path)
            if func is not None:
                func(environ, key, val, overwrite=overwrite)


def _process_env(
    env_file: str,
    search_path: List[Path],
    environ: MutableMapping[str, str],
    overwrite: bool,
    parents: bool,
    errors: bool,
    working_dirs: bool,
    decrypt: bool,
    password: Optional[str] = None,
    encoding: str = DEFAULT_ENCODING,
) -> MutableMapping[str, str]:
    """
    search for any env_files in the given dir list and populate environ dict

    :param env_file: base environment file name to use
    :param search_path: one or more paths to search
    :param environ: environment to update
    :param overwrite: whether to overwrite existing values
    :param parents: whether to search upwards until a file is found
    :param decrypt: whether to attempt decryption
    :param errors: whether to raise FileNotFoundError if the env_file is not found
    :param working_dirs: whether to add the env file's directory
    :param encoding: text encoding
    """
    files_not_found = []
    files_found = False
    for env_path in _env_files(env_file, search_path, parents, decrypt, errors):
        # insert PWD as container of the env file
        env_path = Path(env_path).resolve()
        if working_dirs:
            environ["PWD"] = str(env_path.parent)
        try:
            with open_env(env_path) as f:
                data = f.read()
                if isinstance(data, str):
                    data = data.encode(encoding)
                load_stream(
                    BytesIO(data),
                    environ,
                    overwrite,
                    errors,
                    decrypt,
                    password,
                    encoding,
                    env_path,
                )
            files_found = True
        except FileNotFoundError:
            files_not_found.append(env_path)
    if errors and not files_found and files_not_found:
        raise FileNotFoundError(
            f"{env_file} as {[s.as_posix() for s in files_not_found]}"
        )
    return environ


def _post_process(environ: MutableMapping[str, str]) -> MutableMapping[str, str]:
    """post-process the variables using ${substitutions}"""
    for env_key, env_val in environ.items():
        if all(v in env_val for v in ("${", "}")):  # looks like template
            # ignore anything that does not resolve, don't throw an exception!
            # todo: handle colon separators similar to shell handling..
            #  e.g. PATH=${VALUE:+$VALUE:}some_value
            #  ${VALUE:-default}, ${VALUE:=default}
            val = Template(env_val).safe_substitute(environ)
            if val != env_val:  # don't update unless we need to
                environ[env_key] = val
    return environ


def _update_os_env(environ: MutableMapping[str, str]) -> MutableMapping[str, str]:
    """back-populate changed variables to the environment"""
    for env_key, env_val in environ.items():
        if env_val != os.environ.get(env_key):
            os.environ[env_key] = env_val
    return os.environ


def load_env(
    env_file: str = None,
    search_path: Union[None, Union[List[str], List[Path]], str] = None,
    environ: MutableMapping[str, str] = None,
    overwrite: bool = False,
    parents: bool = False,
    update: bool = True,
    errors: bool = False,
    working_dirs: bool = True,
    decrypt: bool = False,
    password: str = None,
    encoding: Optional[str] = DEFAULT_ENCODING,
) -> MutableMapping[str, str]:
    """
    Loads one or more .env files with optional nesting, updating os.environ
    :param env_file: name of the environment file (.env or $ENV default)
    :param search_path: single or list of directories in order of precedence - str, bytes or Path
    :param environ: environment mapping to process
    :param overwrite: whether to overwrite existing values
    :param parents: whether to search upwards until a file is found
    :param update: option to update os.environ, default=True
    :param errors: whether to raise FileNotFoundError if env_file not found
    :param working_dirs: whether to add the env file's directory
    :param decrypt: whether to support encrypted .env.enc
    :param password: decryption password
    :param encoding: text encoding (default utf-8)
    :returns the new environment
    """
    if environ is None:
        environ = os.environ
    if not env_file:
        env_file = environ.get(DEFAULT_ENVKEY, DEFAULT_DOTENV)

    # insert this as a useful default
    if working_dirs:
        environ["CWD"] = Path.cwd().resolve(strict=True).as_posix()

    # determine where to search
    if search_path is None:
        import inspect

        frame = inspect.stack()[1]
        search_path = [".", frame.filename]
    elif isinstance(search_path, Path):
        search_path = [search_path]
    elif isinstance(search_path, (str, bytes)):
        search_path = search_path.split(os.pathsep)
    # convert to the array of Path for use internally
    search_path = [Path(p) for p in search_path]
    # if overwriting, traverse the path in reverse order so first .env files have priority
    if overwrite:
        search_path.reverse()

    # slurp up the environment files found and
    # post-process values for template variables
    environ = _post_process(
        _process_env(
            env_file,
            search_path,
            environ.copy(),
            overwrite,
            parents,
            errors,
            working_dirs,
            decrypt,
            password,
            encoding,
        )
    )
    # optionally update the actual environment
    return _update_os_env(environ) if update else environ


def load_stream(
    stream: Union[BytesIO, TextIOBase],
    environ: MutableMapping[str, str] = None,
    overwrite: bool = False,
    errors: bool = False,
    decrypt: bool = False,
    password: Optional[str] = None,
    encoding: Optional[str] = DEFAULT_ENCODING,
    env_path: Optional[Path] = None,
):
    if isinstance(stream, TextIOBase):
        stream.seek(0)
        stream = BytesIO(stream.read().encode(encoding))
    elif password and decrypt:
        with contextlib.suppress(DecryptError):
            stream = decrypt_data(stream, password)
    _process_stream(stream, environ, overwrite, errors, encoding, env_path)


load_dotenv = load_env



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Seal or [-u] unseal a vault
"""

import logging
import os
import textwrap

import hvac

description = """\
Seal or [-u] unseal a vault
Uses environment variables:
  VAULT_ADDR - vault server url
  VAULT_CACERT - optional path to CA public certificate
  VAULT_TOKEN  - vault token with sufficient privilege
  VAULT_UNSEAL_KEYS <value,value,vault> - keys to unseal (if unsealing)
"""


def expand(path: str):
    return os.path.expandvars(os.path.expanduser(path))


def trim_indent(text):
    lines = text.splitlines()
    if len(lines) > 0:
        leading_spaces = len(lines[0]) - len(lines[0].lstrip())
        trimmed_lines = [line[leading_spaces:] for line in lines]
        return "\n".join(trimmed_lines)
    return text


def main():
    import argparse

    class CustomFormatter(argparse.RawDescriptionHelpFormatter):
        def _split_lines(self, text, width):
            return textwrap.wrap(text, width=250)

    parser = argparse.ArgumentParser(
        description=description, formatter_class=CustomFormatter
    )
    parser.add_argument(
        "-a",
        "--address",
        default=os.getenv("VAULT_ADDR"),
        help="Override vault url/address",
    )
    parser.add_argument(
        "-t",
        "--token",
        default=os.getenv("VAULT_TOKEN"),
        help="Override token for vault access",
    )
    parser.add_argument(
        "-C",
        "--cacert",
        default=os.getenv("VAULT_CACERT"),
        help="Override path to CA cert key",
    )
    parser.add_argument(
        "-k",
        "--keys",
        default=os.getenv("VAULT_UNSEAL_KEYS"),
        help="Comma separated list of unseal keys",
    )
    action_group = parser.add_mutually_exclusive_group(required=False)
    action_group.add_argument(
        "-s", "--seal", action="store_true", default=False, help="Seal the vault"
    )
    action_group.add_argument(
        "-u", "--unseal", action="store_true", default=False, help="Unseal the vault"
    )

    args = parser.parse_args()

    client = hvac.Client(
        url=args.address, token=args.token, verify=expand(args.cacert) or False
    )

    try:
        if args.seal:
            response = client.sys.seal()
            if response.status_code >= 300:
                response.raise_for_status()

        if args.unseal:
            keys = args.keys.split(",")
            if len(keys) < int(client.seal_status.get("t")):
                raise ValueError(f'Require at least {client.seal_status.get("t")} keys')

            client.sys.submit_unseal_keys(keys=keys)

        status = client.seal_status
        print(
            trim_indent(
                f"Vault Status: {'Sealed' if status['sealed'] == 'true' else 'Unsealed'} "
                f"type={status['type']} shares={status['t']}/{status['n']}"
            )
        )

    except Exception as e:
        logging.error(f"{e.__class__.__name__}: {e}")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Type smart wrapper around os.environ
"""

import contextlib
import re
from pathlib import Path
from io import TextIOBase, BytesIO
from typing import Any, List, MutableMapping, Type

from envex.env_hvac import SecretsManager

from .dot_env import load_env, unquote, load_stream, update_env


class Env:
    """
    Wrapper around os.environ with .env enhancement` and django support
    """

    _exception: Type[Exception]

    _BOOLEAN_TRUE_STRINGS = ("T", "t", "1", "on", "ok", "Y", "y", "en")
    _BOOLEAN_TRUE_BYTES = (b"T", b"t", b"1", b"on", b"ok", b"Y", b"y", b"en")
    _EXCEPTION_CLS = KeyError

    def __init__(
        self,
        *args,
        environ: MutableMapping[str, str] = None,
        exception: Type[Exception] | None = None,
        readenv: bool = False,
        url: str = None,
        token: str = None,
        cert=None,
        verify: bool = True,
        base_path: str = None,
        engine: str | None = None,
        mount_point: str | None = None,
        **kwargs,
    ):
        """

        @param args: dict (optional) environment variables
        @param environ: dict | None default base environment (os.environ is default)
        @param exception: (optional) Exception class to raise on error (default=KeyError)
        @param readenv: read values from .env files (default=False)
        @param decrypt: attempt decryption if password is used
        @param encoding: text encoding
        @param password: password to use for decryption
            - if readenv=True, the following additional args may be used
            @param env_file: str name of the environment file (.env or $ENV default)
            @param search_path: None | Union[List[str], List[Path]], str] path(s) to search for env_file
            @param overwrite: bool whether to overwrite existing environment variables (default=False)
            @param parents: bool whether to search parent directories for env_file (default=False)
            @param update: bool whether to update os.environ with values from env_file (default=False)
            @param errors: bool whether to raise error on missing env_file (default=False)
            - kwargs for vault/secrets manager:
        @param url: (optional) vault url, default is $VAULT_ADDR
        @param token: (optional) vault token, default is $VAULT_TOKEN or ~/.vault-token
        @param cert: (optional) tuple (cert, key) or str path to client cert/key files
        @param verify: (optional) bool | str whether to verify server cert or set ca cert path (default=True)
        @param base_path: (optional) str base path for secrets (default=None)
        @param engine: (optional) str vault secrets engine (default=None)
        @param mount_point: (optional) str vault secrets mount point (default=None, determined by engine)
        @param working_dirs: (optional) bool whether to include PWD/CWD (default=True)
            -
        @param kwargs: (optional) environment variables to add/override
        """
        self._env = self.os_env() if environ is None else environ

        streams = []
        for arg in args:
            if isinstance(arg, dict):
                update_env(self._env, arg)
            elif isinstance(arg, (BytesIO, TextIOBase)):
                streams.append(arg)

        self.exception = exception or self._EXCEPTION_CLS

        if "streams" in kwargs and isinstance(kwargs["streams"], (tuple, list)):
            streams.extend(kwargs.pop("streams"))

        password = kwargs.get("password")
        if kwargs.get("decrypt", False) and password:
            if password[0] == "$":  # use environment variable (pre-.env) - '$' prefix e.g. `$VAR` reads from VAR
                password = self._env.pop(password[1:], None)  # also remove it from the env
            elif password[0] == "@":  # read password from a file - '@' prefix e.g. `@/file/path`
                pw_file = Path(password[1:])
                try:
                    password = pw_file.read_text().rstrip()
                except (IOError, PermissionError) as exc:
                    raise self.exception(*exc.args) from exc
        if not password:
            kwargs["decrypt"] = False
        else:
            kwargs["password"] = password

        kwargs.setdefault("environ", self._env)
        if readenv:
            self.read_env(**kwargs)
        self.read_streams(*streams, **kwargs)
        self.env_source = self.env.get("ENVEX_SOURCE", "env") == "env"
        self.secret_manager = SecretsManager(
            url=url,
            token=token,
            cert=cert,
            verify=verify
            if verify is not None
            else not self.env.get("VAULT_SKIP_VERIFY", False),
            base_path=base_path,
            engine=engine,
            mount_point=mount_point,
            timeout=kwargs.get("timeout", None),
        )

    @staticmethod
    def os_env():
        import os

        return os.environ

    def read_env(self, **kwargs):
        """
        :param kwargs: see load_env
            env_file: str
            search_path: Union[None, Union[List[str], List[Path]], str]
            overwrite: bool
            parents: bool
            update: bool
            errors: bool
            decrypt: bool
            password: str
            encoding: str
        kwargs: MutableMapping[str, str]
        """
        self._env = load_env(**kwargs)

    def read_streams(self, *streams, **kwargs):
        environ = kwargs["environ"]
        # default overwrite is different for streams
        overwrite = kwargs.get("overwrite", True)
        errors = kwargs.get("errors", False)
        decrypt = kwargs.get("decrypt", True)
        password = kwargs.get("password", None)
        encoding = kwargs.get("encoding", "utf-8")
        for stream in streams:
            load_stream(stream, environ, overwrite, errors, decrypt, password, encoding)

    @property
    def exception(self) -> Type[Exception]:
        return self._exception

    @exception.setter
    def exception(self, exc: Type[Exception]):
        self._exception = exc

    @property
    def env(self):
        return self._env

    def get(self, var: str, default=None):
        # getting from the environment is the least expensive
        value = self.env.get(var, None)
        # not set or isn't primary, check secrets manager
        if value is None or not self.env_source:
            sm_value = self.secret_manager.get_secret(var, None)
            if sm_value is not None:
                value = sm_value
        return default if value is None else value

    def pop(self, var, default=None):
        val = self.get(var, default)
        self.unset(var)
        return val

    def set(self, var: str | dict, value=None):
        if isinstance(var, dict):
            for k, v in var.items():
                self.set(k, v)
        else:
            self.env[var] = str(value) if value is not None else value

    def setdefault(self, var, value) -> str | None:
        return self.env.setdefault(var, str(value) if value is not None else value)

    def unset(self, var):
        if var in self.env:
            del self.env[var]

    def is_set(self, var):
        return var in self.env

    def is_all_set(self, *_vars: str | List[str | list | tuple]):
        for v in _vars:
            if isinstance(v, (list, tuple)):
                return self.is_all_set(*v)
            if not self.is_set(v):
                return False
        return True

    def is_any_set(self, *_vars: str | List[str | list | tuple]):
        for v in _vars:
            if isinstance(v, (list, tuple)):
                return self.is_any_set(*v)
            if self.is_set(v):
                return True
        return False

    def int(self, var, default: int | None = None) -> int:
        val = self.get(var, default)
        return self._int(val)

    def float(self, var, default=None) -> float:
        val = self.get(var, default)
        return self._float(val)

    def bool(self, var, default=None) -> bool:
        val = self.get(var, default)
        return val if isinstance(val, (bool, int)) else self.is_true(val)

    def list(self, var, default=None) -> list:
        val = self.get(var, default)
        return val if isinstance(val, (list, tuple)) else self._list(val)

    env_typemap = {
        "str": get,
        "int": int,
        "bool": bool,
        "float": float,
        "list": list,
    }

    # noinspection PyShadowingBuiltins
    def __call__(self, var, default=None, **kwargs):
        if default is not None and not self.is_set(var):
            self.set(var, default)
        _type = kwargs.get("type", str)
        _type = _type if isinstance(_type, str) else _type.__name__
        with contextlib.suppress(KeyError):
            func = self.env_typemap[_type]
            return func(self, var, default=default)
        return self.get(var, default)

    def export(self, *args, **kwargs):
        import os

        for arg in args:
            if not isinstance(arg, (dict,)):
                raise TypeError(
                    "export() requires either dictionaries or keyword=value pairs"
                )
            kwargs |= dict(arg.items())
        if not args and not kwargs:
            kwargs = self.env
        for k, v in kwargs.items():
            k = str(k)
            try:
                if v is None:
                    self.unset(k)
                    del os.environ[k]
                else:
                    self.set(k, v)
                    os.environ[k] = str(v)
            except KeyError:
                ...

    @classmethod
    def _true_values(cls, val):
        return (
            cls._BOOLEAN_TRUE_STRINGS if isinstance(val, str) else cls._BOOLEAN_TRUE_BYTES
        )

    @classmethod
    def is_true(cls, val):
        if val in (None, False, "", 0, "0"):
            return False
        if not isinstance(val, (str, bytes)):
            return bool(val)
        true_vals = cls._true_values(val)
        return bool(val and any(val.startswith(v) for v in true_vals))

    @classmethod
    def _int(cls, val):
        return (
            val if isinstance(val, int) else int(val) if val and str.isdigit(val) else 0
        )

    @classmethod
    def _float(cls, val):
        return val if isinstance(val, float) else float(val) if val else 0

    @classmethod
    def _list(cls, val):
        return (
            []
            if val is None
            else [unquote(part) for part in re.split(r"\s*,\s*", str(val))]
        )

    def __contains__(self, var):
        return self.get(var, None) is not None

    def __setitem__(self, var: str, value: Any):
        self.set(var, value)

    def __getitem__(self, var):
        if var not in self:
            raise self.exception(f"Key '{var}' not found")
        return self.get(var)

    def __delitem__(self, var):
        self.unset(var)

    def items(self):
        yield from self.env.items()

    def __iter__(self):
        return self.items()

    def check_var(self, var, default=None, raise_error=True):
        if not var:
            url = None
        else:
            url = self.get(var, default=default) if var else default
            if not url and raise_error:
                raise self._exception(f"Expected {var} is not set in environment")
        return "" if url is None else url


env = Env()
